#ifndef __FLASH__
  #define __FLASH__

  typedef enum
  {
    DVBS,
    DVBT,
    DVBC
  }FLASH_TYPE;

  typedef struct
  {
    byte		SatIdx;
    byte		polar;
    word		reserved;
    word		freq;
    word		videoPid;
    word		audioPid;
    word		pcrPid;
  } TYPE_DefaultChannel;

  typedef struct
  {
    byte		SatIdx;			//DVB-s only
    byte		Res1;

    word		TPIdx:10;
    word		TunerNum:2;
    word		DelFlag:1;
    word		CASFlag:1;
    word		LockFlag:1;
    word		SkipFlag:1;

    word		SVCID;
    word		PMTPID;
    word		PCRPID;
    word		VideoPID;
    word		AudioPID;

    byte		RenameFlag:1;
    byte		Res2:5;
    byte		SoundMode:2;

    byte		NVODIdx;
    dword		NameOffset;
    word		ProviderIdx;

    word		LCN;			//DVB-t & DVB-c only
    byte		Res3 [8];
  }TYPE_TvSvc ;

  typedef struct
  {
    char		FavName [12];
    word		ServiceIndex [100];
    byte		Flags [100];
    char		NrOfEntries;
    char		Res1;
  }TYPE_Fav;

  typedef struct
  {
    dword		Money [4];
    dword		Win [4];
    dword		Lose [4];
    dword		exblockClearStage;
    byte		Res1 [16];
  }TYPE_GameData;

  typedef struct
  {
    word		UTCOffset;
    word		SleepTimer;

    char		Res1:3;
    char		GMTCollection:3;	//0=Normal, 1=CAS Only, 2=User Select
    char		Mode:2;			//0=Auto, 1=Manual

    char		Res2:7;
    char		TimeOffset:1;		//0=Manual, 1=Automatic

    char		DST:2;			//0=off, 3=on
    char		Res3:6;

    char		Res4;
  }TYPE_ClockSetup;


  /**********************************************************************************/
  /* DVB-s definitions                                                              */
  /**********************************************************************************/

  typedef struct
  {
    unsigned char	SatIdx;

    unsigned char	Polar:1; // 0=V, 1=H
    unsigned char	Mode:3;
    unsigned char	Res1:4;

    byte		Res2 [2];
    dword		Frq;
    word		SR;
    word		TSID;

    unsigned char	AllowTimeSync:1;
    unsigned char	Res3:7;

    byte		Res4;
    word		NetworkID;
  }TYPE_TpInfoS;

  typedef struct
  {
    byte		Power:1;	// 0=off, 1=on
    byte		SW12V:1;	// 0=off, 1=on
    byte		DiSEqC:6;	// 0=disable, 1..16, 17=MiniA, 18=MiniB

    byte		MotorisedSys:1;
    byte		StabPosition:7;

    word		Universal:1;	// 0=single,1=universal LNB
    word		kHz22:1;	// 22khz switch box
    word		LowFreq :14;

    word		HighFreq;

    byte		Usals:1;
    byte		PosIdx:7;

    char		SkewVer;
    char		SkewHor;

    char		Magic:6;
    char		TunerConnectionType:2;

    byte		DiSEqC11;
    byte		Res1 [5];
  }TYPE_LNBInfoS;

  typedef struct
  {
    word		tpNum;
    word		Res1;
    char 		SatName [16];
    TYPE_LNBInfoS	LNB [2];
    TYPE_DefaultChannel DefaultChannel;
    short 	SatAngle;
    word 		Position;
    byte 		Res2 [6];
  }TYPE_SatInfoS;

  typedef struct
  {
    unsigned char	TunerIndex;

    unsigned char	RecMode:3;
    unsigned char	DemuxChannel:2;
    unsigned char	ManualRecording:2;
    unsigned char	Res1:1;

    unsigned char	SatIndex;

    unsigned char	ServiceType:1;
    unsigned char	ReservationType:7;

    word		Duration;
    word		ServiceIndex;
    dword		StartTime;
    dword		EndTime;
    unsigned char	isRec;
    unsigned char	SetName;		//has to be the same value as isRec
    word		ServiceID;
    unsigned char	FileName [98];
    word		EventID;
    TYPE_TpInfoS	TPInfo;
  }TYPE_TimerS;

  typedef struct
  {
    unsigned char	Res1:2;
    unsigned char	ServiceType:1;		//0=TV, 1=Radio
    unsigned char	ServceIndex:6;

    word		EventID;
    dword		StartTime;
  }TYPE_ReserveEPG;


  /**********************************************************************************/
  /* DVB-t definitions                                                              */
  /**********************************************************************************/

  typedef struct
  {
    word		tpNum;
    char 		SatName [30];
  }TYPE_SatInfoT;

  typedef struct
  {
    word		ChannelNr;
    word		Bandwidth;
    byte		Res1;
    dword		Frequency;
    word		TSID;
    unsigned char	StreamType;		//0=LP Stream, 1=HP Stream
    char		Res2;
    word		NetworkID;
    byte  	Res3 [2];
  }TYPE_TpInfoT;

  typedef struct
  {
    unsigned char	TunerIndex;

    unsigned char	RecMode:3;
    unsigned char	DemuxChannel:2;
    unsigned char	ManualRecording:2;
    unsigned char	Res1:1;

    unsigned char	SatIndex;

    unsigned char	ServiceType:1;
    unsigned char	ReservationType:7;

    word		Duration;
    word		ServiceIndex;
    dword		StartTime;
    dword		EndTime;
    unsigned char	isRec;
    unsigned char	SetName;		//has to be the same value as isRec
    word		ServiceID;
    unsigned char	FileName [98];
    word		EventID;
    TYPE_TpInfoT	TpInfo;
  }TYPE_TimerT;


  /**********************************************************************************/
  /* DVB-c definitions                                                              */
  /**********************************************************************************/

  typedef struct
  {
    word		tpNum;
    char 		Res [30];
  }TYPE_SatInfoC;

  typedef struct
  {
    dword		Frequency;		//[kHz]
    word		SymbolRate;
    word		TSID;
    word		NetworkID;
    char		Modulation;		//0=16QAM, 1=32QAM, 2=64QAM, 3=128QAM, 4=256QAM
    byte  	Res1 [1];
  }TYPE_TpInfoC;

  typedef struct
  {
    unsigned char	TunerIndex;

    unsigned char	RecMode:3;
    unsigned char	DemuxChannel:2;
    unsigned char	ManualRecording:2;
    unsigned char	Res1:1;

    unsigned char	SatIndex;

    unsigned char	ServiceType:1;
    unsigned char	ReservationType:7;

    word		Duration;
    word		ServiceIndex;
    dword		StartTime;
    dword		EndTime;
    unsigned char	isRec;
    unsigned char	SetName;		//has to be the same value as isRec
    word		ServiceID;
    unsigned char	FileName [98];
    word		EventID;
    TYPE_TpInfoC	TPInfo;
  }TYPE_TimerC;


  /**********************************************************************************/
  /* Function declarations                                                          */
  /**********************************************************************************/

  dword FlashInitialize (dword SystemID);
  //FLASH_TYPE FlashGetType (void);
  dword FlashGetBlockStartAddress (dword BlockNumber);

  //dword FlashFindServiceAddress (word ServiceID, word PMTPID, word PCRPID, bool TVService);	//Returns an address to the ServiceTable; -1 if not found
  //dword FlashServiceAddressToServiceIndex (dword ServiceAddress);				//Converts the address from the above function to an idx

  //dword FlashFindEndOfServiceTableAddress(bool TVService);
  //int   FlashFindTransponderIndex (dword SatIndex, dword NetworkID, dword TSID);
  //dword FlashFindEndOfServiceNameTableAddress (void);
  //dword FlashRemoveCASServices (bool TVService);
  //dword FlashRemoveServiceByLCN (char* LCN, bool TVService);
  //dword FlashRemoveServiceByName (char* ChannelName, bool TVService);
  //dword FlashRemoveServiceByUHF (char* UHFChannel, bool TVService, bool UseAUSChannelCoding);
  //dword FlashRemoveServiceByIndex (int ChannelIndex, bool TVService);
  //dword FlashRemoveServiceByIndexString (char* ChannelIndex, bool TVService);
  //void  FlashProgram (void);

  //bool FlashAddService (dword Frequency, dword ServiceID, dword PMTPID, dword PCRPID, dword VideoPID, dword AudioPID, char ServiceName [20]);

#endif
