//--------------------------------------------------------------------------
// Program to pull the information out of various types of EXIF digital 
// camera files and show it in a reasonably consistent way
//
// Version 2.48
//
//
// Compiling under Windows:  
//   Make sure you have microsoft's compiler on the path, then run make.bat
//
// Dec 1999 - Jan 2006
//
// by Matthias Wandel   www.sentex.net/~mwandel
//--------------------------------------------------------------------------
#ifdef WIN32
#pragma comment(lib, "c:\\TopfieldDisk\\DataFiles\\SDK\\TAP_SDK_DLL")
#include "TAP_Emulations.h"
#endif

#include "tap.h"

//#include <stdio.h>
//#include <stdlib.h>
//#include <stdlib.h>
//#include <string.h>
//#include <time.h>
//#include <sys/stat.h>
//#include <errno.h>
//#include <ctype.h>


#define JHEAD_VERSION "2.5"

#include "jheadinc.c"

static int FilesMatched;
static int FileSequence;

static const char * CurrentFile;

static const char * progname;   // program name for error messages

//--------------------------------------------------------------------------
// Command line options flags
static int TrimExif = FALSE;        // Cut off exif beyond interesting data.
static int RenameToDate = FALSE;
static char * strftime_args = NULL; // Format for new file name.
static int Exif2FileTime  = FALSE;
static int DoModify     = FALSE;
static int DoReadAction = FALSE;
       int ShowTags     = FALSE;    // Do not show raw by default.
       int DumpExifMap  = FALSE;
static int ShowConcise  = FALSE;
static char * ApplyCommand = NULL;  // Apply this command to all images.
static char * FilterModel = NULL;
static int    ExifOnly    = FALSE;
static int    PortraitOnly = 0;
//static time_t ExifTimeAdjust = 0;   // Timezone adjust
//static time_t ExifTimeSet = 0;      // Set exif time to a value.

static int DeleteComments = FALSE;
static int DeleteExif = FALSE;
static int DeleteUnknown = FALSE;
static char * ThumbSaveName = NULL; // If not NULL, use this string to make up
                                    // the filename to store the thumbnail to.

static char * ThumbInsertName = NULL; // If not NULL, use this string to make up
                                    // the filename to retrieve the thumbnail from.

static int RegenThumbnail = 0;

static char * ExifXferScrFile = NULL;// Extract Exif header from this file, and
                                    // put it into the Jpegs processed.

static int EditComment = FALSE;     // Invoke an editor for editing the comment
static int SupressNonFatalErrors = FALSE; // Wether or not to pint warnings on recoverable errors

static char * CommentSavefileName = NULL; // Save comment to this file.
static char * CommentInsertfileName = NULL; // Insert comment from this file.
static char * CommentInsertLiteral = NULL;  // Insert this comment (from command line)

static int AutoRotate = 0;
static int ZeroRotateTagOnly = 0;

static int ShowFileInfo = 1;        // Indicates to show standard file info
                                    // (file name, file size, file date)


//--------------------------------------------------------------------------
// check if this file should be skipped based on contents.
//--------------------------------------------------------------------------
static int CheckFileSkip(void)
{
    if (ExifOnly){
        // Filtering by EXIF only.  Skip all files that have no Exif.
        if (FindSection(M_EXIF) == NULL){
            return TRUE;
        }
    }

    if (PortraitOnly == 1){
        if (ImageInfo.Width > ImageInfo.Height) return TRUE;
    }

    if (PortraitOnly == -1){
        if (ImageInfo.Width < ImageInfo.Height) return TRUE;
    }

    return FALSE;
}

//--------------------------------------------------------------------------
// Do selected operations to one file at a time.
//--------------------------------------------------------------------------
void ProcessFile(char * FileName)
{
    int Modified = FALSE;
    ReadMode_t ReadMode = READ_EXIF;
    CurrentFile = FileName;
    FilesMatched = 1; 

    // Start with an empty image information structure.
    memset(&ImageInfo, 0, sizeof(ImageInfo));
    ImageInfo.FlashUsed = -1;
    ImageInfo.MeteringMode = -1;
    ImageInfo.Whitebalance = -1;

    strncpy(ImageInfo.FileName, FileName, PATH_MAX);

    if (!ReadJpegFile(FileName, READ_EXIF)) return;

    if (CheckFileSkip()){
        DiscardData();
        return;
    }
    
	ShowImageInfo();

    DiscardData();
    return;
}

//--------------------------------------------------------------------------
// Save exif thumbnail to named file
//--------------------------------------------------------------------------
int SaveThumbnail(char * FileName, char * ThumbFileName)
{
	if (!ReadJpegFile(FileName, READ_EXIF)) return FALSE;

    if (CheckFileSkip()){
        DiscardData();
        return;
    }

	return SaveThumbnailData(ThumbFileName);
}

/** get the pointer to the thumbnail data (if it exists)
    can we assume that the thumbnail is a 160 x 120 JPEG? 
	
	WARNING: remember to call DiscardData when you are finished with the thumbnail, or free the mem yourself
*/ 
char *getThumbnail(char * FileName)
{
	if (!ReadJpegFile(FileName, READ_EXIF)) return FALSE;

    if (CheckFileSkip()){
        DiscardData();
        return;
    }

	return (char *) getThumbnailData();
}