/************************************************************
				Part of the UK TAP Project
		This module handles the loading of the logo files

Name	: logo.c
Author	: Darkmatter
Version	: 0.5
For		: Topfield TF5x00 series PVRs
Licence	:
Descr.	:
Usage	: Uses logos.dat file pre-processed by MakeLogos.
		  Makelogos originates from code written by IanP for the UK TAP project
		  and gif utilities by Topfield & Gershon Elber
History	: v0.0 Darkmatter: 06-06-05	Inception date
		  v0.1 Darkmatter: 08-06-05	Added routing to free memory on exit.
									Add : improved box response during loading of logos
		  v0.2 Darkmatter: 15-06-05	Dramatic speed increase by pre-compiling all logos in to a single logo.dat file
		  v0.3 Darkmatter: 02-07-05	Restore current working directory after use
		  v0.4 Darkmatter: 06-07-05	definition of FormatNameString made void (was undefined)
		  v0.5 Darkmatter: 26-07-05	Fixed problem where long 1st words weren't being displayed until 2nd line.
		  v0.6 Kidhazy:    21-08-05 Updated logo matching to allow matching on LCN as well as Service Name.
		  v0.7 Kidhazy:    22-08-05 New functions to allow for new virtual channel that selects last file that was played.
		  v0.8 Kidhazy:    13-09-05 New function to print name of the selected channel at the top of logos.

	Last change:  USE   7 Aug 105   11:01 pm
************************************************************/

#ifdef WIN32
#pragma comment(lib, "c:\\TopfieldDisk\\DataFiles\\SDK\\TAP_SDK_DLL")
#include "TAP_Emulations.h"
#endif

#include "tap.h"
#include "menu.h"
#include "dlmalloc.h"

#define MAX_SERVICES	2000

// Following 2 definitions moved to DisplayLogosConfig.c
#define LOGO_W	60														// predefined logo width
#define LOGO_H	39														// and height

#define NUM_BYTES	2													// number of bytes per pixel
#define LOGO_DATA_SIZE	LOGO_W * LOGO_H * NUM_BYTES						// number of bytes per logo (bitmap, so nice, fixed size)
#define CHANNEL_NAME_SIZE MAX_SvcName									// bytes allocated the the channel's name

typedef struct															// how we store the logo data
{
	byte	svcName[CHANNEL_NAME_SIZE];
	byte	fileName[CHANNEL_NAME_SIZE];
	word    svcLCN;                     //KH match logos on LCN.
	word	svcNum;
	byte	svcType;
	bool	processedFlag;
	byte	*logo;
} TYPE_logoArray;

static TYPE_logoArray logoArrayTv[MAX_SERVICES/2];						// reserve space for all the TV serivces
static TYPE_logoArray logoArrayRadio[MAX_SERVICES/2];					// and radio services

static int maxTvSvc;
static int maxRadioSvc;
static int logoIndex;
static dword systemTickAtLastKey;

void CreateFileName( TYPE_logoArray *localPtr );          //KH Prototype for later.

void logText(const char* text, bool close_log);
extern char log_buffer[512];
bool isTF5800();
bool ChangeDirProgramFiles();

static unsigned int VALID[8] = {0x00002601,0xFFFFFFFF,0xFFFFFFFF,0x7FFFFFFF,0x00000000,0x00000000,0xFF7FFFFF,0xFF7FFFFF};
#define VALIDBIT(i) (VALID[(i)>>5]&(1<<((i)&0x1F)))

#define IS_CHAR_VALID(c) ((c) == 0x8a) ? (((c) = 0x0a) && VALIDBIT(c)) : VALIDBIT(c)

//and here is the 'real' code, which copies and therefore removes all special characters, double spaces and converts all 0x8a to 0x0a (sometimes the toppy makes a 0x8a for a LF)

/*
  char *TextChar = DestStr;
  bool bSpace = FALSE;

    while(  ( i < uMaxSourceLnge)
          &&((c = *TextChar = QuelleStr[i]) != 0)
          &&(((dword)TextChar - (dword)(p)) < uMaxDestLnge))
    {
        i++;
        if (c == ' ')
        {
            if (bSpace == FALSE)
                TextChar++;
            bSpace = TRUE;
        } else
        if (IS_CHAR_VALID(c))
        {
            bSpace = FALSE;
            TextChar++;
        }
    }

    *TextChar = 0;      // make sure 0 at the end
*/

//------------
//
void DisplayLogo( word WindowRgn, dword Xcoord, dword Ycoord, byte svcNum, byte svcType, char *channel_name )
{
	char	stripped_channel_name[CHANNEL_NAME_SIZE];
	dword	width;
	int		i;

	TYPE_logoArray	*localPtr = NULL;
	TYPE_TapChInfo	currentChInfo;

	// first off, check a match on the channel name for non UK users (in case of dodgy service numbers on Sat boxes)
	if (!isTF5800()) {
		stripPunctuation(channel_name, stripped_channel_name);
		for (i = 0; i < maxTvSvc; i++) {
			if ((strcmp(logoArrayTv[i].fileName, stripped_channel_name) == 0)) {
				if (write_log) if (verbose) TAP_SPrint(log_buffer, "Matched %s by name, number %d", stripped_channel_name, logoArrayTv[i].svcNum);
			 	if (write_log) if (verbose) logText(log_buffer, FALSE);
				localPtr = &logoArrayTv[i];
			}
		}
		if (!localPtr) {
			for (i = 0; i < maxRadioSvc; i++) {
				if ((strcmp(logoArrayRadio[i].fileName, stripped_channel_name) == 0))
					localPtr = &logoArrayRadio[i];
			}
		}
	}

    // if not matched, then ensure service exists and get from array if it does
	if (!localPtr) {
		if (!TAP_Channel_GetInfo(svcType, svcNum, &currentChInfo)) {
			if (write_log) TAP_SPrint(log_buffer, "No service found for %s, type %d, number %d", channel_name, svcType, svcNum);
			if (write_log) logText(log_buffer, FALSE);
			return;	
		}

		if (svcType == SVC_TYPE_Tv) localPtr = &logoArrayTv[svcNum];
		else localPtr = &logoArrayRadio[svcNum];
	}

	if (localPtr->processedFlag == TRUE) {	
		//if (write_log) if (verbose) TAP_SPrint(log_buffer, "Logo found for %s, type %d, number %d", channel_name, svcType, svcNum);
		//if (write_log) if (verbose) logText(log_buffer, FALSE);
		TAP_Osd_RestoreBox(WindowRgn, Xcoord, Ycoord, LOGO_W, 30, localPtr->logo);
	}
	else {	
		if (write_log) TAP_SPrint(log_buffer, "No logo found for %s, type %d, number %d", channel_name, svcType, svcNum);
		if (write_log) logText(log_buffer, FALSE);
	}
}

// filter out punctuation from channel name, also filter special characters (for some German channels)
stripPunctuation(char *source, char *target)
{
	int len, i, j;
	bool found_space = FALSE;
	int c;

	len = strlen(source);
	for (i = j = 0; i < len; i++) {

		c = source[i];
		if (source[i] == ' ') {

			// ignore double spaces
			if (!found_space) {
				target[j] = source[i];
				j++;
			}

            found_space = TRUE;
        } 
		else {
			// ignore special chars & punctuation
			if (IS_CHAR_VALID(c)) {
				if (!ispunct(c)) {
					found_space = FALSE;
					target[j] = source[i];
					j++;
				}
			}
		}
	}

	// terminate the new string
	target[j] = 0;
}

//------------
//
void CreateFileName( TYPE_logoArray *localPtr )
{
	//int len, c, cf;
	char tmp[CHANNEL_NAME_SIZE];

	stripPunctuation(localPtr->svcName, tmp);

	strcpy(localPtr->fileName, tmp);								// save the file name
}



//------------
//
void InitLogoMemory( void )
{
	int i, count;
    int countTvSvc, countRadioSvc;
	TYPE_TapChInfo	currentChInfo;

	TAP_Channel_GetTotalNum( &countTvSvc, &countRadioSvc );				// how many services are defined on this box?

	if ( countTvSvc > MAX_SERVICES/2 )
	{                                                                   // must quit if too many services
//	    ErrorPrint("Too many services");                                // TF5800 won't be affected	- so hack ok for us 
//		return;															// temp - must fix.
		countTvSvc = MAX_SERVICES/2;
	}
	if ( countRadioSvc > MAX_SERVICES/2 )
	{
		countRadioSvc = MAX_SERVICES/2;
	}

	//if (write_log) TAP_SPrint(log_buffer, "Allocate mem for %d TV channels, %d radio channels", countTvSvc, countRadioSvc);
	//if (write_log) logText(log_buffer, FALSE);

	for ( i=0; i<countTvSvc; i++)										// for each TV service
	{
		TAP_Channel_GetInfo( SVC_TYPE_Tv, i, &currentChInfo );			// get the info pointer
		strcpy( logoArrayTv[i].svcName, currentChInfo.chName);			// and copy the channel's name

		logoArrayTv[i].svcLCN = currentChInfo.logicalChNum;				// service LCN
		logoArrayTv[i].svcNum = i;										// service number
		logoArrayTv[i].svcType = SVC_TYPE_Tv;							// service type (TV, or Radio)
		logoArrayTv[i].processedFlag = FALSE;							// no logo cached yet
		//logoArrayTv[i].logo = dlmalloc( LOGO_DATA_SIZE );			// allocate space for the logo
		CreateFileName( &logoArrayTv[i] );								// filter out puntuation and generate a file name
	}

	for ( i=0; i<countRadioSvc; i++)									// do the same for each radio service on this box
	{
		TAP_Channel_GetInfo( SVC_TYPE_Radio, i, &currentChInfo );
		strcpy( logoArrayRadio[i].svcName, currentChInfo.chName);

		logoArrayRadio[i].svcLCN = currentChInfo.logicalChNum;				// service LCN
		logoArrayRadio[i].svcNum = i;
		logoArrayRadio[i].svcType = SVC_TYPE_Radio;
		logoArrayRadio[i].processedFlag = FALSE;
		//logoArrayRadio[i].logo = dlmalloc( LOGO_DATA_SIZE );
		CreateFileName( &logoArrayRadio[i] );							// filter out puntuation and generate a file name
	}

	maxTvSvc = countTvSvc;												// save the number of services for later
	maxRadioSvc = countRadioSvc;

	logoIndex = 0;
}




//------------
//
void ReleaseLogoMemory()
{
    int i;
	
	for ( i=0; i<maxTvSvc; i++)
	{
		if (logoArrayTv[i].logo)
			dlfree( logoArrayTv[i].logo );
	}

	for ( i=0; i<maxRadioSvc; i++)
	{
		if (logoArrayRadio[i].logo)
			dlfree( logoArrayRadio[i].logo );
	}

	dlmalloc_trim(0);
}



//------------
//
bool findChannelPointer( char *channelName, TYPE_logoArray **localPtr )
{
	char stripped_channel_name[CHANNEL_NAME_SIZE];
    int i;
    char LCNstr[10];     //KH temportay storage of LCN as a string.

	// for foreign models, strip the channel name of punctuation and special chars (incase of +, ! etc.)
	stripPunctuation(channelName, stripped_channel_name);

	for ( i=0; i<maxRadioSvc; i++)										// then compare all the Radio services
	{
        sprintf(LCNstr, "%d", logoArrayRadio[i].svcLCN);       //KH Convert integer LCN to string to allow comparison to channel 'filename'

		if (( strcmp( logoArrayRadio[i].fileName, stripped_channel_name ) == 0) ||
     		(strcmp( LCNstr, channelName ) == 0 ))                     //KH we match logos based on service name or LCN
		{
			*localPtr = &logoArrayRadio[i];
			return TRUE;
		}
	}
	for ( i=0; i<maxTvSvc; i++)										// first compare all the TV services
	{
        sprintf(LCNstr, "%d", logoArrayTv[i].svcLCN);       //KH Convert integer LCN to string to allow comparison to channel 'filename'

		if (( strcmp( logoArrayTv[i].fileName, stripped_channel_name ) == 0 ) ||
             (strcmp( LCNstr, channelName ) == 0 )) 		             //KH we match logos based on service name or LCN
		{
			*localPtr = &logoArrayTv[i];
			//if (write_log) TAP_SPrint(log_buffer, "Logo %s loaded and matched, type %d, number %d", logoArrayTv[i].fileName, logoArrayTv[i].svcType, logoArrayTv[i].svcNum);
			//if (write_log) logText(log_buffer, FALSE);
			return TRUE;
		}
	}

	if (write_log) if (verbose) TAP_SPrint(log_buffer, "Logo %s loaded but not matched, filtered name: %s", channelName, stripped_channel_name);
	if (write_log) if (verbose) logText(log_buffer, FALSE);
	return FALSE;														// didn't find any service name that match
}



//------------
//
void ReadLogoFile()
{
	TYPE_File	*logoFile;
	TYPE_logoArray *logoPointer;
	dword		dataSize, file_size, file_tell;
	byte		*junkBuffer;
	char		channelName[CHANNEL_NAME_SIZE];
	int i; 

	ChangeDirProgramFiles();
	if (!TAP_Hdd_Exist("smlogo.dat")) {
		ChangeDirLogos();
		if (!TAP_Hdd_Exist("smlogo.dat")) return;
	}

	logoFile = TAP_Hdd_Fopen("smlogo.dat");
	if (logoFile == NULL) return;

	//if (write_log) logText("Start to read in logos", FALSE);
		
	//file_size = TAP_Hdd_Flen(logoFile);
	//if ((file_size > 0) && (file_size % 512) == 0)
	//	file_size += 512;

    junkBuffer = dlmalloc(LOGO_DATA_SIZE);								// create a place to read in unwanted logos
		
	while (TAP_Hdd_Ftell(logoFile) < TAP_Hdd_Flen(logoFile))
	{
		TAP_Hdd_Fread( channelName, CHANNEL_NAME_SIZE, 1, logoFile);		// read the channel name
		if (findChannelPointer( channelName, &logoPointer))					// cross referece the array to find this entry
		{
			logoPointer->logo = dlmalloc(LOGO_DATA_SIZE);
			TAP_Hdd_Fread(logoPointer->logo, LOGO_DATA_SIZE, 1, logoFile);	// read the logo bitmap
			logoPointer->processedFlag = TRUE;								// mark this channel as having a logo in the cache
		}
		else
		{																	// else channel doesn't appear in service list ->
			TAP_Hdd_Fread(junkBuffer, LOGO_DATA_SIZE, 1, logoFile);			// must read the logo, otherwise read pointer will get stuck
		}
	}

	TAP_Hdd_Fclose(logoFile);
	dlfree(junkBuffer);
	dlmalloc_trim(0);

	//if (write_log) logText("Finish reading in logos", FALSE);
}
	

//------------
//
void CacheLogos( void )
{
	if (write_log) logText("Load logos", FALSE);
	InitLogoMemory();
	ReadLogoFile();
}

