/*
Copyright Pavel Kopylov

v. 1.08

26.09.2007
*/

#ifdef WIN32
#pragma comment(lib, "c:\\TopfieldDisk\\DataFiles\\SDK\\TAP_SDK_DLL")
#include "TAP_Emulations.h"
#endif

#include "tap.h"
#include "rasterfontbrowser.h"
#include "dlmalloc.h"
#include "menu.h"
//#include <string.h>
//#include <stdio.h>


#define htonl(A) ((((long)(A) & 0xff000000) >> 24) | (((long)(A) & 0x00ff0000) >> 8)  | (((long)(A) & 0x0000ff00) << 8)  | (((long)(A) & 0x000000ff) << 24)    )
#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))

#ifdef HOOK
extern FONT FONT_SMALL;
extern FONT FONT_MEDIUM;
extern FONT FONT_BIG;
#endif // HOOK
 
int Load_Font_Height(FONT *font);
int Load_Font_Baseline(FONT *font);
int Load_Char_Table(FONT* font);
void FillImage(FONT *font, word *image, int width, int height);
void ComputeAntialiasingMap(FONT *font, word foreColor, word backColor);
int CalcSize(FONT *font, unsigned char* text, int* width, int *text_length);
int RenderString(FONT *font, unsigned char* text, int text_length, int width, word** image);

void draw_bitmap(FONT *font, unsigned char* bitmap, unsigned char *image, int width, int height, int x_offset, int y_offset, int bitmap_rows, int bitmap_width);


//////////////////////////////////////////////////////////////////////////
int TAP_Osd_PutS_Font(word rgn, dword x, dword y, dword maxX, char *str, word fcolor, word bcolor, FONT *font, byte align)
{
    int err=0;


    SetColors(font, fcolor, bcolor);
    err = TAP_Osd_PutS_FontL(rgn, x, y, maxX, str, font, align);

    return err;
};


//////////////////////////////////////////////////////////////////////////
int TAP_Osd_PutS_FontEx(word rgn, dword x, dword y, dword maxX, dword maxY, int baseline_shift, char *str, word fcolor, word bcolor, FONT *font, byte align)
{
    int err=0;


    font->m_fontHeight = max((int)maxY, font->m_fontHeight);
    if (font->m_bShifted==0)
    {
        font->m_fontBaseline += baseline_shift;
        font->m_bShifted=1;
    }
    err = TAP_Osd_PutS_Font(rgn, x, y, maxX, str, fcolor, bcolor, font, align);

    return err;
};

//////////////////////////////////////////////////////////////////////////
int TAP_Osd_PutS_FontL(word rgn, dword x, dword y, dword maxX, char *str, FONT *font, byte align)
{
    int new_x;
    int width;
    int strlen_original, strlen_new;
    int err=0;
    word *image=0;

    width = (maxX == 0) ? 0 : maxX - x;

    strlen_original = (int)strlen(str);
    err = CalcSize(font, (unsigned char *)str, &width, &strlen_new);
    if (err != 0)
        return err;

    // Draw background rectangle
    //if (maxX == 0)
    //    TAP_Osd_FillBox(rgn, x, y, width, font->m_fontHeight, font->m_backColor);
    //else
    //    TAP_Osd_FillBox(rgn, x, y, maxX-x, font->m_fontHeight, font->m_backColor);

    // Decision of placement
    if (strlen_new < strlen_original)    // the required area is not enough for "str"
    {
        err = RenderString(font, (unsigned char*)str, strlen_new, width, &image);
        if (err==0)
        {
            TAP_Osd_DrawPixmap(rgn, (int)x, (int)y, width, font->m_fontHeight, image, TRUE, OSD_1555);
#ifndef NO_ALLOC
            dlfree(image);
#endif            
        }
    }
    else
    {
        switch(align)
        {
        case ALIGN_LEFT:
            new_x = (int)x;
            break;
        case ALIGN_CENTER:
            new_x = (int)( (x + maxX)/2 - width/2);
            break;
        case ALIGN_RIGHT:
            new_x = (int)(x + maxX - width);
            break;
        }
        err = RenderString(font, (unsigned char*)str, strlen_original, width, &image);
        if (err==0)
        {
            TAP_Osd_DrawPixmap(rgn, new_x, (int)y, width, font->m_fontHeight, image, TRUE, OSD_1555);
#ifndef NO_ALLOC
            dlfree(image);
#endif            
        }
    }

    return err;
}

//////////////////////////////////////////////////////////////////////////
int Draw_Font_String(word region, dword x, dword y, char *str, char* fontname,  word foreColor, word backColor)
{
    int err1, width=0, text_length;
    FONT font;
    word *image=NULL;


    err1 = Load_Font(&font, fontname);
    if (err1==0)
    {
        SetColors(&font, COLOR_Yellow, COLOR_DarkBlue);
        err1 = CalcSize(&font, (unsigned char *)str, &width, &text_length);
        if (err1==0)
        {
            err1 = RenderString(&font, (unsigned char *)str, text_length, width, &image);
            if (err1==0)
            {
                TAP_Osd_DrawPixmap(region, x, y, width, font.m_fontHeight, image, FALSE, OSD_1555);
#ifndef NO_ALLOC
                dlfree(image);
#endif            
            }
        }
        Delete_Font(&font);
    }

    return err1;
}

//////////////////////////////////////////////////////////////////////////
int Load_Font(FONT *font, char *fileName)
{
    int            ret=-1, read, err;
    TYPE_File    *f;
    long        flen;
    //char        msg[256];

    
#ifdef NO_ALLOC
    font->m_grayBuffer = NULL;
    font->m_wordBuffer = NULL;
#endif

    font->m_bShifted = 0;
    f = TAP_Hdd_Fopen(fileName);
    if (f!=0)
    {
        font->m_bmpHeaderArray=NULL;

        flen = TAP_Hdd_Flen(f);
		font->m_data = (unsigned char*)dlmalloc(flen);

        if (font->m_data!=NULL)
        {
            read = TAP_Hdd_Fread(font->m_data, flen, 1, f);
            TAP_Hdd_Fclose(f);
            if (read==1)
            {
                err = Load_Font_Height(font);
                if (err==0)
                {
                    //TAP_SPrint(msg, "height: %d", font->m_fontHeight);
                    //TAP_Osd_PutS(rgn, 50, 500, 150, msg, COLOR_Yellow, COLOR_DarkBlue, 0, FNT_Size_1419, 0, ALIGN_LEFT);
                    err = Load_Font_Baseline(font);
                    if (err==0)
                    {
                        //TAP_SPrint(msg, "baseline: %d", font->m_fontBaseline);
                        //TAP_Osd_PutS(rgn, 180, 500, 280, msg, COLOR_Yellow, COLOR_DarkBlue, 0, FNT_Size_1419, 0, ALIGN_LEFT);
                        err = Load_Char_Table(font);
                        if (err==0)
                        {
#ifdef NO_ALLOC
                            font->m_grayBuffer = (unsigned char*)dlmalloc(720 * font->m_fontHeight * sizeof(unsigned char));
                            font->m_wordBuffer = (word*)dlmalloc(720 * font->m_fontHeight * sizeof(word)); // word

                            if (font->m_grayBuffer==NULL || font->m_wordBuffer==NULL)
                            {
                                return 7;
                            }
#endif
                            ret = 0;
                        } 
                        else //Font table didn't load
                        {
                            ret = 6;
                        }
                    }
                    else //Font baseline not determined
                    {
                        ret = 5;
                    }
                }
                else //Font height not determined
                {
                    ret = 4;
                }
            }
            else //File didn't read
            {
                ret = 3;
            }
        }
        else // not enough memory
        {
            ret = 2;
        }
    }
    else
    {
        // File didn't open
        ret = 1;
    }

    return ret;
}

//////////////////////////////////////////////////////////////////////////
void Delete_Font(FONT *font)
{
#ifdef NO_ALLOC
    if (font->m_grayBuffer != NULL)
    {
		dlfree(font->m_grayBuffer);
    }
    if (font->m_wordBuffer != NULL)
    {
		dlfree(font->m_wordBuffer);
    }
#endif

    if (font != NULL)
    {
		dlfree(font->m_data);
    }

	dlmalloc_trim(0);

}

//////////////////////////////////////////////////////////////////////////
int Load_Font_Height(FONT *font)
{
    int error=1;
    long *ptr;


    if (font!=NULL)
    {
        ptr = (long*)(font->m_data);
        font->m_fontHeight = *ptr;
#ifdef _WIN32
        font->m_fontHeight = htonl(font->m_fontHeight);
#endif
        error=0;
    }

    return error;
}

//////////////////////////////////////////////////////////////////////////
int Load_Font_Baseline(FONT *font)
{
    int error=1;
    long *ptr;


    if (font!=NULL)
    {
        ptr = (long*)(font->m_data+4);
        font->m_fontBaseline = *ptr;
#ifdef _WIN32
        font->m_fontBaseline = htonl(font->m_fontBaseline);
#endif
        error=0;
    }

    return error;
}

//////////////////////////////////////////////////////////////////////////
int Load_Char_Table(FONT* font)
{
    int error=1;
#ifdef _WIN32
    int i;
#endif


    if (font!=NULL)
    {
        font->m_bmpHeaderArray = (TableItem*)(font->m_data+8);
        error=0;
    }

#ifdef _WIN32
    for (i=0; i<256; i++)
    {
        font->m_bmpHeaderArray[i].slot_advance_x = htonl(font->m_bmpHeaderArray[i].slot_advance_x);
        font->m_bmpHeaderArray[i].slot_advance_y = htonl(font->m_bmpHeaderArray[i].slot_advance_y);
        font->m_bmpHeaderArray[i].slot_bitmap_left = htonl(font->m_bmpHeaderArray[i].slot_bitmap_left);
        font->m_bmpHeaderArray[i].slot_bitmap_top = htonl(font->m_bmpHeaderArray[i].slot_bitmap_top);
        font->m_bmpHeaderArray[i].bitmap_rows = htonl(font->m_bmpHeaderArray[i].bitmap_rows);
        font->m_bmpHeaderArray[i].bitmap_width = htonl(font->m_bmpHeaderArray[i].bitmap_width);
        font->m_bmpHeaderArray[i].offset = htonl(font->m_bmpHeaderArray[i].offset);
    }
#endif

    return error;
}


//////////////////////////////////////////////////////////////////////////
void SetColors(FONT *font, word foreColor, word backColor)
{
    if (font->m_foreColor != foreColor || font->m_backColor != backColor)
    {
        font->m_foreColor=foreColor;
        font->m_backColor=backColor;
        ComputeAntialiasingMap(font, foreColor, backColor);
    }
}

//////////////////////////////////////////////////////////////////////////
int CalcSize(FONT *font, unsigned char* text, int* width, int *text_length)
{
    return CalcSizeRounded(font, text, width, text_length, TRUE);
}

int CalcSizeRounded(FONT *font, unsigned char* text, int* width, int *text_length, bool rounded)
{
    int pen_x=0, pen_y=0, i, int_adv, tmp_len;
    //int t_max_bmp_top=0, t_min_bmp_bottom=0;


    if (font==NULL || font->m_bmpHeaderArray==NULL)
    {
        return -1;
    }    

    // First calculating resulting bitmap size & cashing bitmaps
    tmp_len = (int)strlen((char*)text);
    (*text_length) = tmp_len;

    for (i=0; i<tmp_len; i++)
    {
        int_adv = max(font->m_bmpHeaderArray[text[i]].slot_advance_x, font->m_bmpHeaderArray[text[i]].bitmap_width);
		//TAP_Print("Char %c, width %d\r\n", text[i], int_adv);
        if ((*width)>0)
        {
            if (((pen_x + int_adv)>>6)>(*width))
            {
                (*text_length) = i;
                break;
            }
        }
        pen_x += int_adv;

        //t_min_bmp_bottom = max(t_min_bmp_bottom, font->m_bmpHeaderArray[text[i]].bitmap_rows-font->m_bmpHeaderArray[text[i]].slot_bitmap_top);
        //t_max_bmp_top = max(t_max_bmp_top, font->m_bmpHeaderArray[text[i]].slot_bitmap_top);
    }

    (*width) = pen_x>>6;
    if (rounded)
		(*width) = ((*width)/2+1)*2;
    //font->m_fontHeight = max(font->m_fontHeight, t_min_bmp_bottom+t_max_bmp_top);

    return 0;
}

//////////////////////////////////////////////////////////////////////////
int RenderString(FONT *font, unsigned char* text, int text_length, int width, word** image)
{
    int             pen_x=0, pen_y=0, i;
    int             y_offset, x_offset;
    unsigned char    *img;      // whole string
    word            C, *imagePtr;
    int             x, y;
    unsigned char    pixel;
    unsigned char    *bitmap;  // character bitmap


    if (font==NULL || font->m_bmpHeaderArray==NULL)
    {
        return -1;
    }    

#ifdef NO_ALLOC
    img = font->m_grayBuffer;
#else
    // Create bitmap buffer
    img = (unsigned char*)dlmalloc(width * font->m_fontHeight * sizeof(unsigned char));
    if (img==NULL)
    {
        return -2;
    }
#endif
    for (i=0; i<width*font->m_fontHeight; i++) img[i] = 0;

    // Render string to a buffer
    for (i=0; i<text_length; i++)
    {    
        if (i==0)
        {
            pen_x = -font->m_bmpHeaderArray[text[i]].slot_bitmap_left;
            pen_y = font->m_fontBaseline;
        }
        x_offset = pen_x+font->m_bmpHeaderArray[text[i]].slot_bitmap_left;
        y_offset = pen_y-font->m_bmpHeaderArray[text[i]].slot_bitmap_top;

        LOAD_CHAR(font, text[i], &bitmap);
        draw_bitmap(font, bitmap, img, width, font->m_fontHeight, x_offset, y_offset, font->m_bmpHeaderArray[text[i]].bitmap_rows, font->m_bmpHeaderArray[text[i]].bitmap_width);

        pen_x += font->m_bmpHeaderArray[text[i]].slot_advance_x >> 6; // increment pen position
        pen_y += font->m_bmpHeaderArray[text[i]].slot_advance_y >> 6; // increment pen position
    }


#ifdef NO_ALLOC
    (*image) = font->m_wordBuffer;
#else
    (*image) = (word*)dlmalloc(width * font->m_fontHeight * sizeof(word)); // word
    if ((*image)==NULL)
    {
        return -2;
    }
#endif
    imagePtr = (*image);

    // Transfer buffer to the image
    y = font->m_fontHeight*width;
    for (x=0; x<y; x++)
    {
        pixel = img[x];
        C = font->m_antialiasmap[pixel>>4];
        imagePtr[x] = C; 
    }

#ifdef NO_ALLOC
    ;
#else
    dlfree(img);
#endif

    return 0;
}

//////////////////////////////////////////////////////////////////////////
void draw_bitmap(FONT *font, unsigned char* bitmap, unsigned char *image, int width, int height, int x_offset, int y_offset, int bitmap_rows, int bitmap_width)
{
    int  i, j, q;
    int  bx, by;
    unsigned char pixel;


    q=0;
    for (by = 0; by<bitmap_rows; by++)
    {
        j = by + y_offset;
        for (bx = 0; bx<bitmap_width; bx++)
        {
            i = bx + x_offset;
            if (i>=0 && j>=0 && i<width && j<height)
            {
                pixel = bitmap[q];
                PUTPIXEL_OR(image, i, j, pixel); //image[j*width+i] = pixel;
            }
            q++;
        }
    }
}

//////////////////////////////////////////////////////////////////////////
// computes 16 anti-aliasing steps trough RGB space
void ComputeAntialiasingMap(FONT *font, word foreColor, word backColor)
{
    int dimmer=16, i;
    int fr, fg, fb, br, bg, bb;


    if (foreColor!=backColor)
    {
        fr = GETR(foreColor); fg = GETG(foreColor);  fb = GETB(foreColor);
        br = GETR(backColor); bg = GETG(backColor);  bb = GETB(backColor);

        for(i=0; i<16; i++)
        {
            font->m_antialiasmap[i] = RGB( (fr*dimmer+(255-dimmer)*br)>>8, (fg*dimmer+(255-dimmer)*bg)>>8, (fb*dimmer+(255-dimmer)*bb)>>8);
            dimmer += 16;
        }
        font->m_antialiasmap[0] = backColor;
        font->m_antialiasmap[15] = foreColor;
    }
    else
    {
        for(i=0; i<16; i++)
        {
            font->m_antialiasmap[i] = backColor;
        }
    }
}



int getFontIndex(char *name)
{
	int i;

	for (i = 0; i < font_count; i++) {
		if (strcmp(fonts[i].font_name, name) == 0)
			return i;
	}

	return -1;
}

extern char log_buffer[512];

void loadFont(char *name)
{
	int font_result;
	int index ;

	char *temp;
	temp = (char *) strchr(name, '_');
	if (strlen(name) < 11 || temp == NULL) 
		return;

	memset(log_buffer, '\0', 512);
	strncat(log_buffer, name, strlen(name) - strlen(temp));

	index = getFontIndex(log_buffer);

	if (index < 0) {
		memset(&fonts[font_count], 0, sizeof(fonts[font_count]));
		fonts[font_count].font_name = dlmalloc(100 * sizeof(char));

		strcpy(fonts[font_count].font_name, log_buffer);
		index = font_count;
		font_count++;
	}

	if (write_log) TAP_SPrint(log_buffer, "Load font %s", name);
	if (write_log) logText(log_buffer, FALSE);

	if (strlen(name) > 10 && strcmp(name + strlen(name) - 10, "_small.fnt") == 0) {
		fonts[index].small_font = dlmalloc(sizeof(FONT));

		font_result = Load_Font(fonts[index].small_font, name);
	}
	if (strlen(name) > 11 && strcmp(name + strlen(name) - 11, "_medium.fnt") == 0) {
		fonts[index].medium_font = dlmalloc(sizeof(FONT));

		font_result = Load_Font(fonts[index].medium_font, name);
	}
	if (strlen(name) > 10 && strcmp(name + strlen(name) - 10, "_large.fnt") == 0) {
		fonts[index].large_font = dlmalloc(sizeof(FONT));

		font_result = Load_Font(fonts[index].large_font, name);
	}
}

int compareFonts(const void *s1, const void *s2)
{
	font_1 = *(struct _font *) s1;
	font_2 = *(struct _font *) s2;

	if (strcmp(font_1.font_name, "Default") == 0)
		return -1;

	if (strcmp(font_2.font_name, "Default") == 0)
		return 1;

	// if we get here, default to order by name
	return strcmp(font_1.font_name, font_2.font_name);
}

void loadFonts()
{
	TYPE_File file;
	dword i;

	if (write_log) logText("Load fonts", FALSE);

	memset(&fonts[0], 0, sizeof(fonts[0]));
	fonts[0].font_name = dlmalloc(100 * sizeof(char));
	strcpy(fonts[0].font_name, "Default");
	current_font = 0;
	font_count = 1;

#ifndef WIN32
	// if font manager installed, then don't bother loading the fonts
	if (PatchIsInstalled(0x80000000, "Fm")) {
		TAP_Print("Font Manager is running, don't load fonts\n");
		return;
	}
#endif

	if (ChangeDirFonts()) {
		i = TAP_Hdd_FindFirst(&file);
		while(i-- && font_count < 16) {
			if (isFont(file.name))
				loadFont(file.name);

			if (i == 0 || TAP_Hdd_FindNext(&file) == 0)
				break;
		}

		qsort(fonts, font_count, sizeof(fonts[0]), compareFonts);

		// find the idx of the current font 
		current_font = 0;
		for (i = 0; i < font_count; i++) {
			if (strcmp(fonts[i].font_name, font_name) == 0)
				current_font = i;
		}
	}

	loadFont(font_name);
}

void deleteFonts()
{
	word i;

	for (i = 0; i < font_count; i++) {
		if (fonts[i].font_name)
			dlfree(fonts[i].font_name);
		if (fonts[i].small_font) {
			Delete_Font(fonts[i].small_font);
			dlfree(fonts[i].small_font);
		}
		if (fonts[i].medium_font) {
			Delete_Font(fonts[i].medium_font);
			dlfree(fonts[i].medium_font);
		}
		if (fonts[i].large_font) {
			Delete_Font(fonts[i].large_font);
			dlfree(fonts[i].large_font);
		}
	}
}

//////////////////////////////////////////////////////////////////////////
// Hook    for TAP_Osd_GetW
#ifdef HOOK

int TAP_Osd_GetW_HOOK(const char *str, byte fntType, byte fntSize)
{
    int retval=0;
    FONT *font=NULL;


    /*if (fntType!=FONT_OVERRIDE)
    {
        retval = TAP_Osd_GetW(str, fntType, fntSize);
    }
    else
    {*/
		//TAP_Print("Hook GetW %s, %d, %d\r\n", str, fntType, fntSize);

        switch(fntSize)
        {
        case FNT_Size_1419:
            font = fonts[current_font].small_font;
            break;
        case FNT_Size_1622:
            font = fonts[current_font].medium_font;
            break;
        case FNT_Size_1926:
            font = fonts[current_font].large_font;
            break;
        }

        CalcSize(font, (unsigned char*)str, &retval, 0);
    //}

    return retval;
}

//////////////////////////////////////////////////////////////////////////
// Hook    for TAP_Osd_PutS
// fntType : font type. It SHOULD be 0.
// fntSize : font size. FNT_Size_1419, FNT_Size_1622 or FNT_Size_1926
// bDot    : It decides to print  in case that the string is over the maxX. - not used so far
int TAP_Osd_PutS_HOOK(word rgn, dword x, dword y, dword maxX, const char *str, word fcolor, word bcolor, byte fntType, byte fntSize, byte bDot, byte align)
{
    int retval=0;
    FONT *font=NULL;


    /*if (fntType!=FONT_OVERRIDE)
    {
        retval = TAP_Osd_PutS(rgn, x, y, maxX, str, fcolor, bcolor, fntType, fntSize, bDot, align);
    }
    else
    {*/

		//TAP_Print("Hook PutS %s, %d, %d\r\n", str, fntType, fntSize);

        switch(fntSize)
        {
        case FNT_Size_1419:
            font = fonts[current_font].small_font;
            break;
        case FNT_Size_1622:
            font = fonts[current_font].medium_font;
            break;
        case FNT_Size_1926:
            font = fonts[current_font].large_font;
            break;
        }

        retval = TAP_Osd_PutS_Font(rgn, x, y, maxX, (unsigned char *)str, fcolor, bcolor, font, align);
    //}
    
    return retval;
}

#endif //HOOK